/*-------------------------------------------------------------
 *  Copyright (c) 2010-2013, MSS ComBlock. All rights reserved.
 *
 * 	Filename: fpga.h
 *  Last modified: 4/3/13
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 1.0
 *  Inheritance: N/A
 *  Description: Utilities to configure the FPGA
 *  Dependencies/Pre-requisite: N/A
 *
 *  4/3/13 increased FPGA bit file sizes as per the latest ug380 document
 -------------------------------------------------------------*/
#ifndef FPGA_H_
#define FPGA_H_

/*-------- change this section according to the schematics ----------------------------------------*/

// ARM - FPGA communication (while the FPGA is being configured)
#define PIN_FPGA_PROGB_H() 		FIO0SET = 0x00000001;    // port0[0]
#define PIN_FPGA_SUSPEND_H() 	FIO0SET = 0x00000002;	  // port0[1]
#define PIN_FPGA_HSWAPEN_H() 	FIO0SET = 0x00400000;    // port0[22]
#define PIN_FPGA_CCLK_H() 		FIO2SET = 0x00000100;    // port2[8]
#define PIN_FPGA_RWB_H() 		FIO2SET = 0x00000200;    // port2[9]
#define PIN_FPGA_CSIB_H() 		FIO2SET = 0x00000400;    // port2[10]

#define PIN_FPGA_PROGB_L() 		FIO0CLR = 0x00000001;    // port0[0]
#define PIN_FPGA_SUSPEND_L() 	FIO0CLR = 0x00000002;	  // port0[1]
#define PIN_FPGA_HSWAPEN_L() 	FIO0CLR = 0x00400000;    // port0[22]
#define PIN_FPGA_CCLK_L() 		FIO2CLR = 0x00000100;    // port2[8]
#define PIN_FPGA_RWB_L() 		FIO2CLR = 0x00000200;    // port2[9]
#define PIN_FPGA_CSIB_L() 		FIO2CLR = 0x00000400;    // port2[10]

// ARM - FPGA communication (once the FPGA is configured). Pins are being redefined.
#define PIN_FPGA_ALE_H() 		FIO2SET = 0x00000100;    // port2[8]
#define PIN_FPGA_REB_H();		FIO2SET = 0x00000200;    // port2[9]
//#define PIN_FPGA_CSIB_H() 	FIO2SET = 0x00000400;    // port2[10]
#define PIN_FPGA_WEB_H() 		FIO0SET = 0x00000080;    // port0[7]

#define PIN_FPGA_ALE_L() 		FIO2CLR = 0x00000100;    // port2[8]
#define PIN_FPGA_REB_L();		FIO2CLR = 0x00000200;    // port2[9]
//#define PIN_FPGA_CSIB_L() 	FIO2CLR = 0x00000400;    // port2[10]
#define PIN_FPGA_WEB_L() 		FIO0CLR = 0x00000080;    // port0[7]

// Sizes as per ug380  (seems to have increased a bit in the latest version)
// thus add a little 0.5% margin to be safe
// Change comments as a function of the FPGA size LX45
#define FPGA_CONFIG_SIZE	 	1500000 		// (1492412 previously 1484388) XC6SLX45 total number of configuration bytes
//#define FPGA_CONFIG_SIZE	 	4260000		// (4238708 previously 4220196) XC6SLX150 total number of configuration bytes
#define NFBLOCKSPERCONFIG		12			// XC6SLX45 number of 128KB NAND flash blocks per FPGA configuration
//#define NFBLOCKSPERCONFIG		33			// XC6SLX150 number of 128KB NAND flash blocks per FPGA configuration
/*-------- end of schematics-dependent code ----------------------------------------*/

// return values
enum {
	FPGA_OK = 0,
	FPGA_ETIMEOUT,
	FPGA_BADCONFIGNO,
	FPGA_OUTOFRANGE,
} fpga_retvals;

extern void FPGA_NFn(void);
extern void NF_FPGAn(void);
extern void fpga_dataout(unsigned char);
extern long fpga_config(unsigned long);
extern long fpga_readback(void);
extern long fpga_check_initb(void);
extern long fpga_check_done(void);
extern long block_to_config(unsigned long);
extern void fpga_reset(void);
extern unsigned char fpga_getByte(unsigned long);
extern void fpga_setByte(unsigned long , unsigned char);
extern long fpga_config_exists(unsigned long);

#endif /* FPGA_H_ */
