/*-------------------------------------------------------------
 *  Copyright (c) 2010, MSS ComBlock. All rights reserved.
 *
 * 	Filename: nandFlash.h
 *  Last modified: 2/26/12
 *  Authors:
 *  	Alain Zarembowitch / MSS
 *  Version: 2.0
 *  Inheritance: N/A
 *
 *  Rev 2 2/26/12 AZ
 *  Added define statement to allow for 2,4 and 8Gb NAND flash ICs
 ------------------------------------------------------------*/

#ifndef NANDFLASH_H_
#define NANDFLASH_H_

/* change according to installed NAND flash size */
#define MT29F2_4_8G  0	// 0 for 1Gb, non-zero for 2,4 or 8Gbit

#define NBLOCKS  1024
// Number of blocks used. MT29F1Gxx 1Gb -> 1024 blocks (each block is 128K + 4K bytes)
// actual number of blocks used. We may run out of ARM RAM memory if trying to use more than 2048 blocks
#define BLOCKSIZE 131072	// block size in bytes
#define PAGESIZE 2048	// page size in bytes

/*-------- change this section according to the schematics ----------------------------------------*/
#define PIN_NAND_WPB_H() FIO1SET = 0x04000000;    // port1[26]
#define PIN_NAND_CLE_H() FIO1SET = 0x10000000;	  // port1[28]
#define PIN_NAND_CEB_H() FIO1SET = 0x20000000;    // port1[29]
#define PIN_NAND_ALE_H() FIO2SET = 0x00000100;    // port2[8]
#define PIN_NAND_REB_H() FIO2SET = 0x00000200;    // port2[9]
#define PIN_NAND_WEB_H() FIO1SET = 0x00008000;    // port1[15]

#define PIN_NAND_WPB_L() FIO1CLR = 0x04000000;    // port1[26]
#define PIN_NAND_CLE_L() FIO1CLR = 0x10000000;	  // port1[28]
#define PIN_NAND_CEB_L() FIO1CLR = 0x20000000;    // port1[29]
#define PIN_NAND_ALE_L() FIO2CLR = 0x00000100;    // port2[8]
#define PIN_NAND_REB_L() FIO2CLR = 0x00000200;    // port2[9]
#define PIN_NAND_WEB_L() FIO1CLR = 0x00008000;    // port1[15]
/*-------- end of schematics-dependent code ----------------------------------------*/

// return values
enum {
	NF_OK = 0,
	NF_ETIMEOUT,
	NF_EBUSY,
	NF_EBADBLOCK,
	NF_EDATAIN,
	NF_ETOOMANYBADBLOCKS,	// number of detected bad blocks exceeds the manufacturer-specified limit (2,008 (NVB) valid blocks	out of every 2,048 total available blocks)
} nf_retvals;


// NAND flash info
struct nandFlash_info {
	// information read from the NAND flash itself (read ID operation)
	unsigned char manufacturerID;
	unsigned char deviceID;
	unsigned char byte3;  // see specs for breakdown (page size, etc)
	// blocks re-direction after initialization. When block is OK, re-direction = index
	unsigned long blockRedirect[NBLOCKS];
	unsigned long nBadBlocks;	// total number of bad blocks


};

// variables
extern struct nandFlash_info nfInfo;

// function prototypes
extern long nandFlash_init(void);
extern long nandFlash_readID(void);
extern long nandFlash_blockErase(unsigned long blockNo);
extern long nandFlash_reset(void);
extern unsigned char nandFlash_check_status(void);
extern unsigned char nandFlash_check_status_contd(void);
extern long nandFlash_page_read(unsigned long, unsigned long, unsigned long, unsigned long, unsigned char *);
extern long nandFlash_checkBadBlock(unsigned long);
extern long nandFlash_setBadBlock(unsigned long);
extern long nandFlash_page_program_start(unsigned long, unsigned long, unsigned long, unsigned long, char *);
extern long nandFlash_program_contd(unsigned long, unsigned long, char *);
extern long nandFlash_page_program_end(void);
extern long nandFlash_read(unsigned long, unsigned char *, unsigned long);
extern inline void busDirOut(void);
extern inline void busDirIn(void);
extern inline unsigned char busDataIn(void);
extern inline void busDataOut(unsigned char);

// higher level user functions
extern long nandFlash_EFS(unsigned long);
extern long nandFlash_GCS(unsigned long, unsigned long *);

#endif /* NANDFLASH_H_ */
